require 'spec_helper'

describe Rtpl::Parser do
  let(:parser) { Rtpl::Parser.new }
  let(:f)      { fixtures }

  describe 'spaces' do
    it 'should parse one' do
      parser.space.should parse(' ')
    end

    it 'should parse multiple' do
      parser.space.should parse('   ')
    end
  end

  describe 'spc?' do
    it 'should parse empty string' do
      parser.spc?.should parse('')
    end

    it 'should parse one space' do
      parser.spc?.should parse(' ')
    end

    it 'should parse multiple spaces' do
      parser.spc?.should parse('  ')
    end
  end

  describe 'name' do
    it 'should parse a block_name' do
      parser.name.should parse('main')
    end

    it 'should parse a longer block_name' do
      parser.name.should parse('block_name')
    end

    it 'should not accept whitespace' do
      parser.name.should_not parse('name   ')
    end
  end

  describe 'open_sym' do
    it 'should parse an open_sym string with whitespace' do
      parser.open_sym.should parse('  BEGIN:  ')
    end

    it 'should parse an open_sym string without whitespace' do
      parser.open_sym.should parse('BEGIN:')
    end

    it 'should require a colon' do
      parser.open_sym.should_not parse('BEGIN')
    end
  end

  describe 'close_sym' do
    it 'should parse a close_sym string with whitespace' do
      parser.close_sym.should parse('  END:  ')
    end

    it 'should parse a close_sym string without whitespace' do
      parser.close_sym.should parse('END:')
    end

    it 'should require a colon' do
      parser.close_sym.should_not parse('END')
    end
  end

  describe 'comment' do
    it 'should parse a comment' do
      parser.comment.should parse('<!--')
    end

    it 'should parse a comment followed by whitespace' do
      parser.comment.should parse('<!--  ')
    end
  end

  describe 'end_comment' do
    it 'should parse an end_comment' do
      parser.end_comment.should parse('-->')
    end

    it 'should parse an end_comment preceded by whitespace' do
      parser.end_comment.should parse('  -->')
    end
  end

  describe 'open_block' do
    it 'should parse an open_block' do
      parser.open_block.should parse('<!-- BEGIN: block_name-->')
    end
  end

  describe 'close_block' do
    it 'should parse an close_block' do
      parser.close_block.should parse('<!-- END: block_name-->')
    end
  end

  describe 'block' do
    subject { Rtpl::Parser.new.block }

    it { should parse(f.empty_block) }
    it { should parse(f.block_with_plaintext) }
    it { should parse(f.block_with_variable) }
    it { should parse(f.block_with_variables) }
    it { should parse(f.block_with_html_comment) }
    it { should parse(f.block_with_nested_block) }
    it { should parse(f.block_with_nested_blocks) }
    it { should parse(f.block_with_deep_nested_block) }
    it { should parse(f.block_with_deep_nested_blocks) }

    it { should_not parse(f.open_block_without_end) }
    it { should_not parse(f.end_block_without_open) }
  end

  describe 'nested_block' do
    it 'should parse a nested_block' do
      parser.nested_block.should parse(f.nested_block)
    end
  end
end
