require 'spec_helper'

describe Rtpl do
  let(:f)                  { fixtures }
  let(:rtpl)               { Rtpl.new(f.block_with_plaintext) }
  let(:rtpl_nested)        { Rtpl.new(f.block_with_nested_block) }
  let(:rtpl_nested_nested) { Rtpl.new(f.block_with_nested_nested_block) }
  let(:sample2)            { Rtpl.new(f.jtpl_sample2)}

  describe 'assign' do

    it 'should alter the vars hash' do
      rtpl.assign("k","v")

      rtpl.vars.should == {"k" => "v"}
    end
  end

  describe 'parse' do
    it 'should parse a top level block and add it to parsed_blocks' do
      rtpl.parse("block_name")

      rtpl.parsed_blocks["block_name"].should == "the content of the block\n"
    end

    it 'should parse a block repeatedly and  accumulate parsed_blocks' do
      2.times { rtpl.parse("block_name") }

      rtpl.parsed_blocks["block_name"].should ==
        "the content of the block\nthe content of the block\n"
    end

    it 'should parse a sub_block' do
      rtpl_nested.parse("block_name.nested")

      rtpl_nested.parsed_blocks["block_name.nested"].should ==
        "the content of the block\n"
    end

    it 'should parse a sub_block then a top level block pushing text up' do
      rtpl_nested.parse("block_name.nested")
      rtpl_nested.parse("block_name")

      rtpl_nested.parsed_blocks["block_name"].should ==
        "before_text\nthe content of the block\nafter_text\n"
    end

    it 'should parse a nested nested_block' do
      rtpl_nested_nested.parse("block_name.nested.nested2")
      rtpl_nested_nested.parse("block_name.nested")
      rtpl_nested_nested.parse("block_name").should ==
        "before_text\nbefore_text\nthe content of the block\nafter_text\nafter_text\n"

    end

    it 'should error when trying to parse a nonexistent nested_block' do
      lambda { rtpl_nested.parse("block_name.z") }.should raise_error ArgumentError
    end

    it 'should error when trying to parse a nonexistent block' do
      lambda { rtpl.parse("z") }.should raise_error ArgumentError
      lambda { rtpl.parse("") }.should raise_error ArgumentError
    end
  end

  describe 'out' do
    it 'should take a key and return a string' do
      rtpl_nested.parse("block_name.nested")
      rtpl_nested.parse("block_name")

      rtpl_nested.out("block_name").should ==
        "before_text\nthe content of the block\nafter_text\n"
    end
  end

  describe 'jtpl samples' do
    it 'should match the output from jtpl' do
      output = File.open(File.join(File.dirname(__FILE__), "fixtures", "jtpl_sample2.html"))
      sample2.assign('TITLE', 'Sample 2')

      10.times do |i|
        5.times do |j|
          sample2.assign('BGCOLOR', j % 2 == 0 ? '#75D2FF' : '#CCEEFF')
          sample2.assign('VARIABLE', "line:#{i + 1} col:#{j + 1}")
          sample2.parse("main.line.row")
        end

        sample2.parse("main.line")
      end

      sample2.parse("main")
      sample2.out.should == output.read
    end
  end
end
